/*
 * SWatchFSM2.c
 *
 *  Created on: 11/nov/2012
 *      Author: Marco
 */

#include "SWatchFSM2.h"
#include "Watch.h"
#include "LCDdisplay.h"

FSMWatch myFSM;

Signal DecodeEvents(unsigned char events)
{
	if (events >= 0x08) return MODE;
	if ((events & 0x04) == 0x04) return SUBMODE;
	if ((events & 0x01) == 0x01) return PLUS;
	if ((events & 0x02) == 0x02) return MINUS;
	return NOEVENT;
}

void SWatchFSMDispatch(unsigned char events)
{
	switch(myFSM.curState) {
	case DISPLAYTIME:
		switch(DecodeEvents(events)) {
		case MODE:
			// Actions
			// State transition
			SWatchFSMTran(SETTIMEHRS);
			LCDMode(DMODETIMESET);
			break;
		default:
			LCDDisplayTime(&myFSM.watch);
			break;
		}
		break;
	case SETTIMEHRS:
		switch(DecodeEvents(events)) {
		case MODE:
			// Actions
			// State transition
			SWatchFSMTran(SETALARMHRS);
			LCDDisplayAlarm(&myFSM.watch);
			LCDMode(DMODEALARM);
			break;
		case SUBMODE:
			// Actions
			// State transition
			SWatchFSMTran(SETTIMEMINS);
			LCDDisplayTime(&myFSM.watch);
			break;
		case PLUS:
			// Actions
			ChangeTime(&myFSM.watch.time, (char)1, (char)0);
			LCDDisplayTime(&myFSM.watch);
			// State transition
			break;
		case MINUS:
			// Actions
			ChangeTime(&myFSM.watch.time, (char)-1, (char)0);
			LCDDisplayTime(&myFSM.watch);
			// State transition
			break;
		default:
			LCDDisplayTime(&myFSM.watch);
			break;
		}
		break;
		case SETTIMEMINS:
		switch(DecodeEvents(events)) {
		case MODE:
			// Actions
			// State transition
			SWatchFSMTran(SETALARMHRS);
			LCDDisplayAlarm(&myFSM.watch);
			LCDMode(DMODEALARM);
			break;
		case SUBMODE:
			// Actions
			// State transition
			SWatchFSMTran(SETTIMEHRS);
			LCDDisplayTime(&myFSM.watch);
			break;
		case PLUS:
			// Actions
			ChangeTime(&myFSM.watch.time, (char)0, (char)1);
			LCDDisplayTime(&myFSM.watch);
			// State transition
			break;
		case MINUS:
			// Actions
			ChangeTime(&myFSM.watch.time, (char)0, (char)-1);
			LCDDisplayTime(&myFSM.watch);
			// State transition
			break;
		default:
			LCDDisplayTime(&myFSM.watch);
			break;
		}
		break;
	case SETALARMHRS:
		switch(DecodeEvents(events)) {
		case MODE:
			// Transition Actions
			// Exit Actions
			// State transition
			SWatchFSMTran(CHRONOSTOP);
			// Entry Actions
			LCDDisplayChrono(&myFSM.watch);
			LCDMode(DMODECHRONO);
			break;
		case SUBMODE:
			// Actions
			// State transition
			SWatchFSMTran(SETALARMMINS);
			LCDDisplayAlarm(&myFSM.watch);
			break;
		case PLUS:
			// Actions
			ChangeTime(&myFSM.watch.alarm, (char)1, (char)0);
			LCDDisplayAlarm(&myFSM.watch);
			// State transition
			break;
		case MINUS:
			// Actions
			ChangeTime(&myFSM.watch.alarm, (char)-1, (char)0);
			LCDDisplayTime(&myFSM.watch);
			// State transition
			break;
		default:
			LCDDisplayAlarm(&myFSM.watch);
			break;
			}
		break;
		case SETALARMMINS:
		switch(DecodeEvents(events)) {
		case MODE:
			// Transition Actions
			// Exit Actions
			// State transition
			SWatchFSMTran(CHRONOSTOP);
			// Entry Actions
			LCDDisplayChrono(&myFSM.watch);
			LCDMode(DMODECHRONO);
			break;
		case SUBMODE:
			// Actions
			// State transition
			SWatchFSMTran(SETALARMHRS);
			LCDDisplayAlarm(&myFSM.watch);
			break;
		case PLUS:
			// Actions
			ChangeTime(&myFSM.watch.alarm, (char)0, (char)1);
			LCDDisplayAlarm(&myFSM.watch);
			// State transition
			break;
		case MINUS:
			// Actions
			ChangeTime(&myFSM.watch.alarm, (char)0, (char)-1);
			LCDDisplayTime(&myFSM.watch);
			// State transition
			break;
		default:
			LCDDisplayAlarm(&myFSM.watch);
			break;
			}
		break;
	case CHRONOSTOP:
		switch(DecodeEvents(events)) {
		case MODE:
			// Transition Actions
			// Exit Actions
			// State transition
			SWatchFSMTran(DISPLAYTIME);
			// Entry Actions
			LCDDisplayTime(&myFSM.watch);
			LCDMode(DMODETIME);
			break;
		case CHSTART:
			// Transition Actions
			// Exit Actions
			// State transition
			SWatchFSMTran(CHRONORUN);
			// Entry Actions
			break;
		case CHRESET:
			// Transition Actions
			InitTime(&myFSM.watch.chrono);
			LCDDisplayTime(&myFSM.watch);
			// Exit Actions
			// State transition
			// Entry Actions
			break;
		default:
			LCDDisplayChrono(&myFSM.watch);
			break;
			}
		break;
		case CHRONORUN:
		switch(DecodeEvents(events)) {
		case MODE:
			// Transition Actions
			// Exit Actions
			// State transition
			SWatchFSMTran(DISPLAYTIME);
			// Entry Actions
			LCDDisplayTime(&myFSM.watch);
			LCDMode(DMODETIME);
			break;
		case CHSTOP:
			// Transition Actions
			// Exit Actions
			// State transition
			SWatchFSMTran(CHRONOSTOP);
			// Entry Actions
			break;
		default:
			TickTenth(&myFSM.watch.chrono);
			LCDDisplayChrono(&myFSM.watch);
			break;
			}
		break;
	}
}

WatchType *SWatchFSMGetWatch()
{
	return &(myFSM.watch);
}

void SWatchFSMInit()
{
	myFSM.curState = DISPLAYTIME;
	InitWatch(&myFSM.watch);
}

void SWatchFSMTran(State nextstate)
{
	myFSM.curState = nextstate;
}

